referrerpolicy=no-referrer-when-downgrade

DynamicMaxBlockWeight

Struct DynamicMaxBlockWeight 

pub struct DynamicMaxBlockWeight<Config, Inner, TargetBlockRate, const MAX_TRANSACTION_TO_CONSIDER: u32 = 10, const ALLOW_NORMAL: bool = true>(pub Inner, _);
Transaction extension that dynamically changes the max block weight.

With block bundling, parachains are running with block weights that may not allow certain transactions to be applied, e.g. a runtime upgrade. To ensure that these transactions can still be applied, this transaction extension can change the max block weight as required. There are multiple requirements for it to change the block weight:

  1. Only the first block of a core is allowed to change its block weight.

  2. Any inherent or any transaction up to MAX_TRANSACTION_TO_CONSIDER requires more block weight than the target extrinsic weight. Target extrinsic weight is the max weight for the respective extrinsic class. The priority to determine the target e weight is the following, we start checking if WeightsPerClass::max_extrinsic is set, after this WeightsPerClass::max_total and if both of these are None we fall back to the actual target block weight.

Because the node is tracking the wall clock time while building a block to abort block production if it takes too long, we do not allow any block to change the block weight. The node knows that the first block of a core may runs longer. So, the node allows this block to take up to 2s of wall clock time. 2s is the time each PoV gets on the relay chain for its validation or in other words the maximum core execution time. The extension sets the [CumulusDigestItem::UseFullCore] digest when the block should occupy the entire core.

Before dispatching an extrinsic the extension will check the requirements and set the appropriate BlockWeightMode. After the extrinsic has finished, the checks from before dispatching the extrinsic are repeated with the post dispatch weights. The BlockWeightMode is changed properly.

Note

The extension requires that any of the inner extensions sets the BlockWeight. Otherwise the weight tracking is not working properly. Normally this is done by CheckWeight.

Generic parameters

  • Config: The Config trait of this pallet.

  • Inner: The inner transaction extensions aka the other transaction extensions to be used by the runtime.

  • TargetBlockRate: The target block rate the parachain should be running with. Or in other words, the number of blocks the parachain should produce in 6s(relay chain slot duration).

  • MAX_TRANSACTION: The maximum number of transactions to consider before giving up to change the max block weight.

  • ALLOW_NORMAL: Should transactions with a dispatch class Normal be allowed to change the max block weight?

Tuple Fields

0: Inner

Implementations

Source

impl<T, S, TargetBlockRate, const MAX_TRANSACTION_TO_CONSIDER: u32, const ALLOW_NORMAL: bool> DynamicMaxBlockWeight<T, S, TargetBlockRate, MAX_TRANSACTION_TO_CONSIDER, ALLOW_NORMAL>

pub fn new(s: S) -> Self

Create a new DynamicMaxBlockWeight instance.